#!/usr/bin/python3
"""
Description: 
Author: MALossov
Date: 2023-01-27 11:44:22
LastEditTime: 2023-01-27 11:48:58
LastEditors: MALossov
Reference: 
"""


import argparse
import os
import ptt_clk_mal.pc_clk as pc_clk
import ptt_clk_mal.pc_his as pc_his
import ptt_clk_mal.pc_set as pc_set
from rich.console import Console

console = Console()


def cli():
    sets = pc_set.Settings()
    if os.path.exists("%s" % pc_set.confYaml):
        sets.load()
    else:
        console.print(
            "[bold red]Warning!: pttsets.yaml does not exist!!Use DEFAULT SETTINGS!"
        )
        console.print(
            "[bold dark_blue]Please run [italic cyan]pttClk set -i[/] to init the settings"
        )
    parser = argparse.ArgumentParser(description="Set the clock on a pttClk router")
    # 参数部分
    parser.add_argument(
        "-t",
        "--time",
        type=int,
        help="Set the time of the clock",
        dest="time",
        required=False,
        default=sets.default_time,
    )
    parser.add_argument(
        "-n",
        "--notify",
        type=float,
        help="Notice intervals of the clock",
        dest="Intervals",
        required=False,
        default=sets.default_intervals,
    )
    parser.add_argument(
        "-a" "--aim",
        type=str,
        help="Set the aim of the clock",
        dest="aim",
        required=False,
        default=sets.default_aim,
    )
    parser.add_argument(
        "-c",
        "--color",
        type=str,
        help="Set colors of progress",
        dest="color",
        required=False,
        default=sets.default_color,
    )
    parser.add_argument(
        "-b",
        "--background",
        type=str,
        help="Set colors of progress' backend",
        dest="back",
        required=False,
        default=sets.default_bg,
    )
    parser.add_argument(
        "-p",
        "--pulse",
        type=str,
        help="Set colors of progress' pulse",
        dest="pulse",
        required=False,
        default=sets.default_pulse,
    )
    parser.add_argument(
        "-y",
        "--confirm",
        dest="start",
        action="store_true",
        help="Confirm the autostart啊 of bar",
        default=sets.default_confirm,
    )
    # 子命令部分
    subparsers = parser.add_subparsers(help="sub-commands")

    history_parse = subparsers.add_parser(
        "his", help="show your pttClks' history"
    )
    history_parse.add_argument(
        "-q",
        "--query",
        help="Query the history of pttClk times",
        dest="query",
        type=str,
        default=sets.default_queryT,
        required=False,
    )
    history_parse.add_argument(
        "-c",
        "--clear",
        dest="clear",
        action="store_true",
        help="Clear the history of pttClk times",
    )
    history_parse.add_argument(
        "-g",
        "--groups",
        dest="groups",
        nargs="+",
        type=str,
        required=False,
        default=sets.default_queryG,
        help="Query the history of pttClk times by groups",
    )
    #

    history_parse.set_defaults(handle=history)

    setting_parse = subparsers.add_parser(
        "set", help="Change the settings of default pttClk clocks"
    )
    setting_parse.add_argument(
        "-s", "--show", help="Show the settings", action="store_true"
    )
    setting_parse.add_argument(
        "-c", "--clear", help="Clear the settings", action="store_true"
    )
    setting_parse.add_argument(
        "-i", "--init", help="Init the settings", action="store_true"
    )
    setting_parse.add_argument(
        "-e", "--edit", help="Edit the settings", action="store_true"
    )
    setting_parse.set_defaults(handle=settings)

    args = parser.parse_args()
    if hasattr(args, "handle"):
        args.handle(args)
    else:
        pc_clk.potato_clock(args)


def history(args):
    pc_his.potato_history(args)


def settings(args):
    pc_set.sets(args)



"""
Description: 
Author: MALossov
Date: 2023-01-27 16:51:30
LastEditTime: 2023-01-27 16:57:35
LastEditors: MALossov
Reference: 
"""
import sqlite3
from typing import Any
import os

db_name = os.path.dirname(os.path.abspath(__file__))+"potatoclock.db"


def create_table():
    # 如果当前文件夹下没有potatoclock.db,就创建一个
    conn = sqlite3.connect(db_name)
    # 创建一个游标
    c = conn.cursor()
    # 执行sql语句,创建一个表
    c.execute(
        """CREATE TABLE IF NOT EXISTS pttClk
                    (id INTEGER PRIMARY KEY AUTOINCREMENT,
                    date TEXT,
                    aim TEXT,
                    period INTEGER)"""
    )
    # 提交事务
    conn.commit()
    # 关闭连接
    conn.close()


def insert_potato(pttperiod, pttaim, pttdate):
    # 如果当前文件夹下没有potatoclock.db,就创建一个
    conn = sqlite3.connect(db_name)
    # 创建一个游标
    c = conn.cursor()
    c.execute(
        """INSERT INTO pttClk (period,aim,date)
                    VALUES (?,?,?)""",
        (pttperiod, pttaim, pttdate),
    )
    # 提交事务
    conn.commit()
    # 关闭连接
    conn.close()


def query_factory(sql: str) -> object:
    """

    :rtype: object
    """
    # 如果当前文件夹下没有potatoclock.db,就创建一个
    conn = sqlite3.connect(db_name)
    # 创建一个游标
    c = conn.cursor()
    c.execute(sql)
    # 提交事务
    conn.commit()
    # 关闭连接
    data: list[Any] = c.fetchall()
    conn.close()
    return data


def query_potato():
    return query_factory("""SELECT * FROM pttClk ORDER BY date DESC""")


def query_by_groups(groups: list):
    # merge list into SQL DML string in 'or', which column is 'aim'
    return query_factory(
        "SELECT * FROM pttClk WHERE aim = '"
        + "' or aim = '".join(groups)
        + "' ORDER BY date DESC"
    )


def query_last_xdays(x: int):
    return query_factory(
        """SELECT * FROM pttClk WHERE date > date('now','-{} day') ORDER BY date DESC""".format(
            x
        )
    )


def query_xdays_and_groups(x: int, groups: list):
    return query_factory(
        """SELECT * FROM pttClk WHERE date > date('now','-{} day') AND (aim = '{}""".format(
            x, "' or aim = '".join(groups)
        )
        + "') ORDER BY date DESC"
    )


def clean_table():
    conn = sqlite3.connect(db_name)
    # 创建一个游标
    c = conn.cursor()
    c.execute("""DELETE FROM pttClk WHERE TRUE""")
    # 提交事务
    conn.commit()
    # 关闭连接
    conn.close()


if __name__ == "__main__":
    print("Aborted!")



"""
Description: 
Author: MALossov
Date: 2023-01-27 16:53:44
LastEditTime: 2023-01-27 23:26:31
LastEditors: MALossov
Reference: 
"""
import rich.progress
from rich.console import Console
import os
import sys
import time
import ptt_clk_mal.pc_dao as pc_dao
import datetime

console = Console()


def potato_clock(args):
    # 首先打印当前时间,预计结束的时间,定时的时间和提醒的间隔.
    # 所有时间用不同彩色的字体打印
    # 打印分行符之后打印进度条,进度条的颜色和字体颜色一致
    console.print(
        "Present time:",
        "[yellow underline]" + time.strftime("%H:%M:%S", time.localtime()),
        end="\t\t|\t",
    )
    console.print(
        "Schedule end time:",
        "[blue underline]"
        + time.strftime(
            "%H:%M:%S",
            time.localtime(time.time() + args.time * 60),
        ),
        end="\t|\n",
    )
    console.print(
        "[bold blue]Notify intervals:", args.Intervals, "minutes", end="\t|\t"
    )
    console.print("[bold red]pttClk clock:", args.time, "minutes", end="\t|\n")
    console.rule("[bold blue]Potato Aim:[cyan2]" + args.aim, style="bold cyan")

    # 使用定时器
    # 每隔0.25秒刷新一次进度条
    # 每隔args.Intervals分钟提醒一次
    # 进度条到达100%时,打印完成信息,并退出程序
    with rich.progress.Progress(
        "[progress.description]{task.description}",
        rich.progress.BarColumn(
            bar_width=None,
            style="bold " + args.back,
            complete_style="bold " + args.color,
            pulse_style="bold" + args.pulse,
        ),
        "[progress.percentage]{task.percentage:>3.0f}%",
        rich.progress.TimeRemainingColumn(),
        console=console,
    ) as progress:
        tomatoClkTask = progress.add_task(
            "[bold blue]PotatoClock",
            total=args.time * 60,
            start=False,
        )
        if args.start:
            pass
        else:
            try:
                input("Press [italic magenta]ANY-KEY/Ctrl+C [/]to start/absorb:")
                console.rule(
                    "\r[magenta]Tasking:{}\t|\t[cyan]Total:{}\t".format(
                        args.aim, args.time
                    )
                )
            except KeyboardInterrupt:
                console.print("[bold red]You have absorbed the pttClk clock!")
                sys.exit(0)
        progress.start_task(tomatoClkTask)
        for i in range(args.time * 60):
            progress.advance(tomatoClkTask)
            time.sleep(1)
            if i % (args.Intervals * 60) == 0:
                # 构建通知字符串
                notify_str = """notify-send \"potatoClock!\" \"You passed one Interval for {} minutes!\r\n Now your time 
                left is: {:.1f} minutes\"""".format(
                    args.Intervals, float(args.time - i / 60)
                )
                # 使用notify-send发送通知
                os.system(notify_str)
    console.rule("[bold green]pttClk clock is over!")
    console.print(
        "[italic blue]Cool!!",
        "\t[yellow]You did a :",
        args.time,
        "[yellow]minutes [bold red]pttClk CLOCK!!\n",
        end="",
    )
    notify_str = """notify-send \"potatoClock!\" \"You have finished your pttClk clock!\r\n Good job!\""""
    os.system(notify_str)
    # 将本次的番茄钟信息存入数据库
    # noinspection PyBroadException
    try:
        pc_dao.create_table()
        pc_dao.insert_potato(args.time, args.aim, datetime.datetime.now())
        console.print("[bold green italic]Time well remember Everything~")
    except Exception:
        console.print("[bold red]Failed to insert data into database!")



"""
Description: 
Author: MALossov
Date: 2023-01-27 16:56:56
LastEditTime: 2023-01-28 01:47:31
LastEditors: MALossov
Reference: 
"""
import ptt_clk_mal.pc_dao as pc_dao
import rich
from rich.console import Console
import datetime
import rich.table

console = Console()


def prettierPrintTable(data: list, days: int, hobby: list):
    # use rich to print a table
    table = rich.table.Table(
        show_header=True,
        header_style="bold magenta",
        title="Potato History",
        title_style="bold magenta italic",
    )
    table.add_column("ID", style="dim sky_blue2", width=4)
    table.add_column("Date", style="dim spring_green1", width=12)
    table.add_column("Time", style="dim spring_green2", width=8)
    table.add_column("Aim", style="dim cyan2", width=10)
    table.add_column("Last", style="dim slate_blue3", width=4)
    sum_time = 0
    for row in data:
        sum_time += row[3]
        # let row[2] be the date&HH:MM from datetime
        Date = datetime.datetime.strptime(row[1][:-7], "%Y-%m-%d %H:%M:%S").strftime(
            "%Y-%m-%d"
        )
        Time = datetime.datetime.strptime(row[1][:-7], "%Y-%m-%d %H:%M:%S").strftime(
            "%H:%M"
        )
        table.add_row(str(row[0]), Date, Time, str(row[2]), str(row[3]))
    console.print(table)
    console.rule(
        "[bold italic wheat4]Common Statistics", style="bold orange2", align="left"
    )
    console.print(
        "Experienced {} times potatoes in total!".format(len(data)),
        style="bold hot_pink3",
        end="\t|\t",
    )
    console.print(
        "Total time is {} mins!".format(sum_time), style="bold deep_sky_blue1", end="\n"
    )
    console.print(
        "Your average is {} mins per time!".format(sum_time // len(data)),
        style="bold green",
        end="\n",
    )
    if days:
        console.rule(
            "[bold italic salmon1]Daily Statistics", style="bold plum2", align="left"
        )
        console.print(
            "Experienced {} days in total!".format(days),
            style="bold dark_olive_green3",
            end="\n",
        )
        console.print(
            "Your average is {} minutes per day!".format(sum_time // len(data)),
            style="bold italic sea_green2",
            end="\t|\t",
        )
        console.print(
            "Your average is {} times per day!".format(len(data) // days),
            style="bold salmon1 italic",
            end="\n",
        )
    if hobby:
        console.rule(
            "[bold italic dark_orange1]Hobby Statistics",
            style="bold dark_orange1",
            align="left",
        )
        # from hobbies create a dict
        hobby_dict = {hobby[i]: 0 for i in range(len(hobby))}
        for row in data:
            hobby_dict[row[2]] += 1
        for key in hobby_dict:
            console.print(
                "{}:{} times".format(key, hobby_dict[key]),
                style="bold dark_slate_gray3",
                end="\t|\t",
            )
        console.print("")
    console.rule(
        "[bold italic cyan1]Enjoy Nice New Days!", style="bold green", align="left"
    )


def potato_history(args):
    if args.clear:
        pc_dao.clean_table()
        console.rule(
            "[bold italic chartreuse2]History cleared!Enjoy Nice New Days!",
            style="bold green",
        )
    elif args.query and args.groups:
        queryGroupsAndArgs(args.query, args.groups)
    elif args.query:
        queryByArgs(args.query)
    elif args.groups:
        queryByGroup(args.groups)
    else:
        console.print(
            "[cyan3 Italic underline]type: ",
            "[bold cyan]<CMD> his -h [/bold cyan]",
            "[cyan3 Italic underline]for help",
        )


# noinspection PyTypeChecker
def queryByArgs(dayCount):
    if dayCount == "today":
        data, dayCount = pc_dao.query_last_xdays(1), 1
    elif dayCount == "week":
        data, dayCount = pc_dao.query_last_xdays(7), 7
    elif dayCount == "month":
        data, dayCount = pc_dao.query_last_xdays(30), 30
    elif dayCount == "all":
        data = pc_dao.query_potato()
        dayCount = len(data)
    else:
        try:
            data = pc_dao.query_last_xdays(int(dayCount))
            dayCount = len(data)
        except ValueError:
            console.print("Error: Invalid query string!!\n", style="bold red")
            console.print(
                "[Bold dark_blue]Your query must be: [Italic cyan]today, yesterday, week, month, all or a number"
            )
            exit(1)
    prettierPrintTable(data, dayCount, None)


def queryGroupsAndArgs(query, group):
    try:
        data = pc_dao.query_xdays_and_groups(int(query), group)
        prettierPrintTable(data, int(query), group)
    except ValueError:
        console.print("Error: Invalid query string!!\n", style="bold red")
        console.print(
            "[Bold dark_blue]Your query must be: {Number} for query & Right Names for {Group}"
        )
        exit(1)


def queryByGroup(group):
    try:
        data = pc_dao.query_by_groups(group)
        # noinspection PyTypeChecker
        prettierPrintTable(data, None, group)
    except ValueError:
        console.print("Error: Invalid query string!!\n", style="bold red")
        console.print("[Bold dark_blue]Your query must be: Right Names for {Group}")
        exit(1)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ptt_clk_mal']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'rich>=13.0.0,<14.0.0']

entry_points = \
{'console_scripts': ['pttClk = ptt_clk_mal.pc_parser:cli']}

setup_kwargs = {
    'name': 'ptt-clk-mal',
    'version': '0.1.1',
    'description': 'A small script help get your time used better',
    'long_description': '# MALoPotatoClock\n\nThis is a simple clock that uses a potato to tell the time.\n\ntry type `<CMD>` to enjoy your journal.',
    'author': 'MALossov',
    'author_email': 'lshy1104@163.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import yaml
from rich.console import Console
import os

console = Console()
confYaml = os.path.dirname(os.path.abspath(__file__)) + "/static/pttsets.yaml"


def sets(args):
    if args.init:
        init_settings()
        exit(0)
    if os.path.exists("%s" % confYaml):
        pass
    else:
        console.print(
            "Error: %s does not exist!!" % confYaml,
            style="bold red",
        )
        console.print(
            "[bold blue2]Please run ",
            "[italic cyan]<CMD> set -i",
            " to init the settings",
        )
        exit(1)

    if args.show:
        show_settings()
    elif args.edit:
        edit_settings()
    elif args.clear:
        clear_settings()
    else:
        console.print("Error: Invalid settings command!!", style="bold red")
        console.print(
            "[bold blue]Your settings command must be: [italic cyan], -s, -c, -i"
        )
        exit(1)


def init_settings():
    # Check if pttsets.yaml exists
    if os.path.exists("%s" % confYaml):
        console.print(
            "Error: pttsets.yaml already exists!!",
            style="bold red",
        )
        console.print(
            "[bold blue]Please run ",
            "[italic cyan]<CMD> set -c",
            "[bold blue] to clear the settings",
        )
        exit(1)
    else:
        with open("%s" % confYaml, "w") as f:
            yaml.dump(
                {
                    "TomatoClocks": {
                        "KeepTime": 25,
                        "NoticingIntervals": 2,
                        "DefaultAim": "Study",
                        "AutoStart": False,
                    },
                    "ProgressColorSets": {
                        "BackgroundColor": "deep_sky_blue4",
                        "PulseColor": "aquamarine3",
                        "CompletedColor": "spring_green1",
                    },
                    "DefaultQuerySets": {
                        "DefaultDays": 7,
                        "DefaultQueryAim": "Study",
                    },
                },
                f,
            )
        console.print("Settings file created successfully!!", style="bold green")


def clear_settings():
    # Check if pttsets.yaml exists
    if os.path.exists("%s" % confYaml):
        os.remove("%s" % confYaml)
        console.print("Settings file cleared successfully!!", style="bold green")
    else:
        console.print(
            "Error: pttsets.yaml does not exist!!\n",
            style="bold red",
        )
        console.print(
            "[Bold blue]Please run [Italic cyan]<CMD> set -i[/] to init the settings"
        )
        exit(1)


def show_settings():
    # Check if pttsets.yaml exists
    if os.path.exists("%s" % confYaml):
        with open("%s" % confYaml, "r") as f:
            data = yaml.safe_load(f)
        console.print(data, style="white on navy_blue")
    else:
        console.print(
            "Error: pttsets.yaml does not exist!!\n",
            style="bold red",
        )
        console.print(
            "[Bold blue]Please run [Italic cyan]<CMD> set -i[/] to init the settings"
        )
        exit(1)


def edit_settings():
    # Check if pttsets.yaml exists
    if os.path.exists("%s" % confYaml):
        with open("%s" % confYaml, "r") as f:
            data = yaml.safe_load(f)
        for item in data:
            console.rule(
                "[italic pale_turquoise1]Editing:[/][bold blue]{}[/]?:".format(item),
                style="bold blue",
                align="left",
            )
            for key in data[item]:
                console.print(
                    "[italic pale_turquoise1]Edit[/][bold blue]{}[/]?:".format(key),
                    end="|",
                )
                console.print(
                    "[bold blue]Current value:[/][bold cyan]{}[/]".format(
                        data[item][key]
                    ),
                    end="|",
                )
                console.print("[bold blue]New value:[/]", end="")
                new_value = input()
                if new_value == "":
                    pass
                else:
                    data[item][key] = new_value
        with open("%s" % confYaml, "w") as f:
            yaml.dump(data, f)
        console.rule(
            "Settings file edited successfully!!", style="bold green", align="left"
        )


def load_settings():
    # Check if pttsets.yaml exists
    if os.path.exists("%s" % confYaml):
        with open("%s" % confYaml, "r") as f:
            data = yaml.safe_load(f)
        return data
    else:
        console.print(
            "Error: pttsets.yaml does not exist!!\n",
            style="bold red",
        )
        console.print(
            "[bold blue]Please run [italic cyan]<CMD> set -i[/] to init the settings"
        )
        exit(1)


class Settings:
    def __init__(self):
        self.default_confirm = False
        self.default_time = 25
        self.default_intervals = 2
        self.default_aim = "Study"
        self.default_color = "spring_green1"
        self.default_bg = "deep_sky_blue2"
        self.default_pulse = "aquamarine3"
        self.default_queryG = None
        self.default_queryT = None

    def load(self):
        conf = load_settings()
        self.default_confirm = conf["TomatoClocks"]["AutoStart"]
        self.default_time = conf["TomatoClocks"]["KeepTime"]
        self.default_intervals = conf["TomatoClocks"]["NoticingIntervals"]
        self.default_aim = conf["TomatoClocks"]["DefaultAim"]
        self.default_color = conf["ProgressColorSets"]["CompletedColor"]
        self.default_bg = conf["ProgressColorSets"]["BackgroundColor"]
        self.default_pulse = conf["ProgressColorSets"]["PulseColor"]
        self.default_queryG = conf["DefaultQuerySets"]["DefaultQueryAim"]
        self.default_queryT = conf["DefaultQuerySets"]["DefaultDays"]



import ptt_clk_mal.pc_clk as pc_clk
import ptt_clk_mal.pc_his as pc_his
import ptt_clk_mal.pc_parser as pc_parser
import ptt_clk_mal.pc_set as pc_set

if __name__ == "__main__":
    pc_parser.cli()



